/*
 *  linux/include/linux/stegfs_fs.h
 *
 * Copyright (C) 1999
 * Andrew McDonald (a.d.mcdonald@bcs.org.uk)
 * St. John's College, University of Cambridge, UK
 *
 *  from
 *
 *  linux/include/linux/ext2_fs.h
 *
 * Copyright (C) 1992, 1993, 1994, 1995
 * Remy Card (card@masi.ibp.fr)
 * Laboratoire MASI - Institut Blaise Pascal
 * Universite Pierre et Marie Curie (Paris VI)
 *
 *  from
 *
 *  linux/include/linux/minix_fs.h
 *
 *  Copyright (C) 1991, 1992  Linus Torvalds
 */

#ifndef _LINUX_STEGFS_FS_H
#define _LINUX_STEGFS_FS_H

/*
 * Use most stuff from standard ext2 filesystem
 */
#include <linux/ext2_fs.h>


/*
 * The stegfs filesystem constants/structures
 */

/*
 * Define STEGFS_DEBUG to produce debug messages
 */
#define STEGFS_DEBUG

/*
 * The stegfs file system version
 */
#define STEGFS_DATE		"1999/01/XX"
#define STEGFS_VERSION		"0.1"

/*
 * Debug code
 */
#ifdef STEGFS_DEBUG
#	define stegfs_debug(f, a...)	{ \
					printk ("stegfs DEBUG (%s, %d): %s:", \
						__FILE__, __LINE__, __FUNCTION__); \
				  	printk (f, ## a); \
					}
#else
#	define stegfs_debug(f, a...)	/**/
#endif


#define STEGFS_MAX_LEVELS        16
#define STEGFS_INO_MASK          0x80000000
#define STEGFS_IS_HID_INO(n)     ((n) & STEGFS_INO_MASK) 
#define STEGFS_MAX_INO_COPIES    28
#define STEGFS_MAX_BLOCK_COPIES  14


#ifdef __KERNEL__
#define STEGFS_SB(sb)     (&((sb)->u.stegfs_sb))
#else
/* Assume that user mode programs are passing in an stegfs superblock, not
 * a kernel struct super_block.  This will allow us to call the feature-test
 * macros from user land. */
#define STEGFS_SB(sb)     (sb)
#endif

/*
 * Structure of an entry in the block table
 */
struct stegfs_btable {

	__u32 magic1;
	__u32 magic2;
	__u32 bchecksum;
	__u32 ino;
	/* __u32 type; - do this in magic2? */
};

/*
 * Structure of a stegfs inode on the disk
 */
struct stegfs_inode {
        __u16   i_mode;         /* File mode */
        __u16   i_uid;          /* Owner Uid */
        __u32   i_size;         /* Size in bytes */
        __u32   i_atime;        /* Access time */
        __u32   i_ctime;        /* Creation time */
        __u32   i_mtime;        /* Modification time */
        __u32   i_dtime;        /* Deletion Time */
        __u16   i_gid;          /* Group Id */
        __u16   i_links_count;  /* Links count */
        __u32   i_blocks;       /* Blocks count */
        __u32   i_flags;        /* File flags */
        union {
                struct {
                        __u32  l_i_reserved1;
                } linux1;
                struct {
                        __u32  h_i_translator;
                } hurd1;
                struct {
                        __u32  m_i_reserved1;
                } masix1;
        } osd1;                         /* OS dependent 1 */
        /*__u32 i_block[EXT2_N_BLOCKS];*//* Pointers to blocks */
        __u32   i_version;      /* File version (for NFS) */
        __u32   i_file_acl;     /* File ACL */
        __u32   i_dir_acl;      /* Directory ACL */
        __u32   i_faddr;        /* Fragment address */
        union {
                struct {
                        __u8    l_i_frag;       /* Fragment number */
                        __u8    l_i_fsize;      /* Fragment size */
                        __u16   i_pad1;
                        __u32   l_i_reserved2[2];
                } linux2;
                struct {
                        __u8    h_i_frag;       /* Fragment number */
                        __u8    h_i_fsize;      /* Fragment size */
                        __u16   h_i_mode_high;
                        __u16   h_i_uid_high;
                        __u16   h_i_gid_high;
                        __u32   h_i_author;
                } hurd2;
                struct {
                        __u8    m_i_frag;       /* Fragment number */
                        __u8    m_i_fsize;      /* Fragment size */
                        __u16   m_pad1;
                        __u32   m_i_reserved2[2];
                } masix2;
        } osd2;                         /* OS dependent 2 */
        __u8    i_icopies;        /* Number of copies of inode */
        __u8    i_bcopies;        /* Number of copies of blocks */
	__u16   i_pad0;
	__u32   i_inode[STEGFS_MAX_INO_COPIES];
        __u32   i_block[EXT2_N_BLOCKS *
		       STEGFS_MAX_BLOCK_COPIES];   /* Pointers to blocks */
};


#ifdef __KERNEL__

/* acl.c */
extern int stegfs_permission (struct inode *, int);

/* balloc.c */
extern int stegfs_group_sparse(int group);
extern int stegfs_new_block (const struct inode *, unsigned long,
			    __u32 *, __u32 *, int *);
extern void stegfs_free_blocks (const struct inode *, unsigned long,
			       unsigned long);
extern unsigned long stegfs_count_free_blocks (struct super_block *);
extern void stegfs_check_blocks_bitmap (struct super_block *);
extern struct ext2_group_desc * stegfs_get_group_desc(struct super_block * sb,
						     unsigned int block_group,
						     struct buffer_head ** bh);

/* bitmap.c */
extern unsigned long stegfs_count_free (struct buffer_head *, unsigned);

/* dir.c */
extern int stegfs_check_dir_entry (const char *, struct inode *,
				  struct ext2_dir_entry_2 *,
				  struct buffer_head *,
				  unsigned long);

/* file.c */
extern int stegfs_read (struct inode *, struct file *, char *, int);
extern int stegfs_write (struct inode *, struct file *, char *, int);

/* fsync.c */
extern int stegfs_sync_file (struct file *, struct dentry *);

/* ialloc.c */
extern struct inode * stegfs_new_inode (const struct inode *, int, int *);
extern void stegfs_free_inode (struct inode *);
extern unsigned long stegfs_count_free_inodes (struct super_block *);
extern void stegfs_check_inodes_bitmap (struct super_block *);

/* inode.c */
extern int stegfs_bmap (struct inode *, int);

extern struct buffer_head * stegfs_getblk (struct inode *, long, int, int *);
extern struct buffer_head * stegfs_bread (struct inode *, int, int, int *);

extern int stegfs_getcluster (struct inode * inode, long block);
extern void stegfs_read_inode (struct inode *);
extern void stegfs_write_inode (struct inode *);
extern void stegfs_put_inode (struct inode *);
extern void stegfs_delete_inode (struct inode *);
extern int stegfs_sync_inode (struct inode *);
extern void stegfs_discard_prealloc (struct inode *);

/* ioctl.c */
extern int stegfs_ioctl (struct inode *, struct file *, unsigned int,
			unsigned long);

/* namei.c */
extern void stegfs_release (struct inode *, struct file *);
extern int stegfs_lookup (struct inode *, struct dentry *);
extern int stegfs_create (struct inode *,struct dentry *,int);
extern int stegfs_mkdir (struct inode *,struct dentry *,int);
extern int stegfs_rmdir (struct inode *,struct dentry *);
extern int stegfs_unlink (struct inode *,struct dentry *);
extern int stegfs_symlink (struct inode *,struct dentry *,const char *);
extern int stegfs_link (struct dentry *, struct inode *, struct dentry *);
extern int stegfs_mknod (struct inode *, struct dentry *, int, int);
extern int stegfs_rename (struct inode *, struct dentry *,
			 struct inode *, struct dentry *);

/* super.c */
extern void stegfs_error (struct super_block *, const char *, const char *, ...)
	__attribute__ ((format (printf, 3, 4)));
extern NORET_TYPE void stegfs_panic (struct super_block *, const char *,
				    const char *, ...)
	__attribute__ ((NORET_AND format (printf, 3, 4)));
extern void stegfs_warning (struct super_block *, const char *, const char *, ...)
	__attribute__ ((format (printf, 3, 4)));
extern void stegfs_put_super (struct super_block *);
extern void stegfs_write_super (struct super_block *);
extern int stegfs_remount (struct super_block *, int *, char *);
extern struct super_block * stegfs_read_super (struct super_block *,void *,int);
extern int init_stegfs_fs(void);
extern int stegfs_statfs (struct super_block *, struct statfs *, int);

/* truncate.c */
extern void stegfs_truncate (struct inode *);

/*
 * Inodes and files operations
 */

/* dir.c */
extern struct inode_operations stegfs_dir_inode_operations;

/* file.c */
extern struct inode_operations stegfs_file_inode_operations;

/* symlink.c */
extern struct inode_operations stegfs_symlink_inode_operations;

#endif	/* __KERNEL__ */

#endif	/* _LINUX_STEGFS_FS_H */
